#!/bin/sh

# based on http://endot.org/2012/02/12/git-subtree-tracking-made-easy/
function setup
{
    # for each path that was subtree merged
    for path in `git log --grep Squashed --oneline | awk '{ print $3 }' | sort | uniq | sed "s/'//g" | sed "s/\/$//g"`; do

        # check to see if the subpath is already in .gittrees
        git config -f .gittrees subtree.$path.url &> /dev/null
        if [ $? -eq 0 ]; then
            echo "$path already configured"
        else

            # look for the most recent commit
            commit=`git log --grep "Squashed '$path/'" --oneline | head -n 1 | awk '{ print $NF }'`
            if [[ $commit =~ '..' ]]; then
                commit=`echo $commit | cut -d . -f 3`
            fi
            echo "last commit for $path is $commit";

            # ask for the git url
            echo "Enter url: "
            read URL

            # record the subtree info
            git config -f .gittrees --unset subtree.$path.url
            git config -f .gittrees --add subtree.$path.url $URL
            git config -f .gittrees --unset subtree.$path.path
            git config -f .gittrees --add subtree.$path.path $path
            git config -f .gittrees --unset subtree.$path.branch
            git config -f .gittrees --add subtree.$path.branch master
        fi

        echo "------------------------------"
    done
}

function pull {
    path=$1
    # make sure it's a subtree that exists
    git config -f .gittrees subtree.$path.url &> /dev/null
    if ! [ $? -eq 0 ]; then
        echo "$path is not a registered subtree"
    else
        url=`git config -f .gittrees subtree.$path.url`
        branch=`git config -f .gittrees subtree.$path.branch`
        git subtree pull --prefix=$path $url $branch --squash

        echo "pulled $path"
    fi
}

function push {
    path=$1
    # make sure it's a subtree that exists
    git config -f .gittrees subtree.$path.url &> /dev/null
    if ! [ $? -eq 0 ]; then
        echo "$path is not a registered subtree"
    else
        url=`git config -f .gittrees subtree.$path.url`
        branch=`git config -f .gittrees subtree.$path.branch`
        git subtree push --prefix=$path $url $branch

        echo "pushed $path"
    fi
}

case "$1" in
    setup )
        setup
        ;;
    pull )
        if [ -n "$2" ]; then
            pull $2
        else
            echo "no subtree specified"
        fi
        ;;
    push )
        if [ -n "$2" ]; then
            push $2
        else
            echo "no subtree specified"
        fi
        ;;
    * )
        echo 'subtree helper
    commands:
    setup -- sets up the .subtree file with existing subtrees
    pull [subtree] -- pulls a subtree from its repo
    push [subtree] -- pushes a subtree to its repo'
        ;;
esac